/***************************************************************************
 *   Copyright (C) 2009, 2010 by Stephane List                             *
 *   slist@lilotux.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "version.h"
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QMessageBox>
#include <QFile>

#include "libavutil/pixfmt.h" // to have PIX_FMT_RGB24


/*
Ok, I had the answer.
Yes, the SPS and PPS embedded inside the SDP must be extracted and sent to the decoder. The sending form is the same as the other nal.

Nathalie

sprop-parameter-sets = Stream properties Parameter-sets
SPS : Sequence parameter set
PPS : picture parameter set

 I always get FU-A packets and now what do i need to decode it? i mean which class or function?


Error while decoding stream #0.0
[h264 @ 015A8560]non-existing PPS referenced
[h264 @ 015A8560]decode_slice_header error
[h264 @ 015A8560]no frame!

http://tools.ietf.org/html/rfc3984
   Fragmentation unit: Used to fragment a single NAL unit over multiple RTP packets
video coding layer (VCL) and a network abstraction layer (NAL).
 The NAL unit type octet has the following format:

      +---------------+
      |0|1|2|3|4|5|6|7|
      +-+-+-+-+-+-+-+-+
      |F|NRI|  Type   |
      +---------------+

  Table 1.  Summary of NAL unit types and their payload structures

      Type   Packet    Type name                        Section
      ---------------------------------------------------------
      0      undefined                                    -
      1-23   NAL unit  Single NAL unit packet per H.264   5.6
      24     STAP-A    Single-time aggregation packet     5.7.1
      25     STAP-B    Single-time aggregation packet     5.7.1
      26     MTAP16    Multi-time aggregation packet      5.7.2
      27     MTAP24    Multi-time aggregation packet      5.7.2
      28     FU-A      Fragmentation unit                 5.8
      29     FU-B      Fragmentation unit                 5.8
      30-31  undefined                                    -

a=fmtp:96 packetization-mode=1; profile-level-id=420029;

sprop-parameter-sets=Z0IAKeNQFAe2AtwEBAaQeJEV,aM48gA==


*/

const char header[] = { 0x00, 0x00, 0x01 };

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow), vd(0)
{
    ui->setupUi(this);
    r = new RtspClientUdp(this);
    connect(r, SIGNAL(connected()), this, SLOT(slot_rtspConnected()));
    connect(r, SIGNAL(disconnected()), this, SLOT(slot_rtspdisConnected()));
    connect(r, SIGNAL(AudioAvailable(const QByteArray&)), this, SLOT(slot_audio(const QByteArray&)));
    connect(r, SIGNAL(VideoAvailable(const QByteArray&)), this, SLOT(slot_video(const QByteArray&)));
    r->connectToHost("192.168.1.72", 554);


    // TODO : remove all this...
    QFile f("bitstream.h264");
    f.resize(0);
    QByteArray Data = QByteArray::fromRawData(header, 3);
    if (!f.open(QIODevice::Append))
    {
        qDebug("can't write to file");
        return;
    }
    f.write(Data.data(), Data.size());
    f.close();
}

MainWindow::~MainWindow()
{
    delete ui;
    delete vd;
}

void MainWindow::slot_rtspConnected()
{
    qDebug() << "PLAY";
    vd = new VideoDecoder(r->get_video_format());
    if (r->get_video_format() == "MP4V-ES")
    {
        vd->set_bitrate(r->get_video_bit_rate());
        vd->mp4_set_profile_id(r->sdp_mpeg4_profile_level_id_);
        vd->mp4_set_global_headers(r->sdp_mpeg4_video_config_);
    }
    else if (r->get_video_format() == "H264")
    {
        vd->set_bitrate(r->get_video_bit_rate());
        vd->h264_set_packetization_mode(r->sdp_h264_packetization_mode_);
        vd->h264_set_profile_level_id(r->sdp_h264_profile_level_id_);
        vd->h264_set_sprop_parameter_sets(r->sdp_h264_sprop_parameter_sets_);
    }
    else
    {
        qWarning("This codec is not supported, perhaps it needs extra parameters");
    }
    vd->init();
    r->play();
}

void MainWindow::slot_rtspdisConnected()
{
    qDebug() << "disconnected";
}

void MainWindow::on_actionAbout_qIpCam_triggered()
{
    QMessageBox::about(this,
                       tr("About"),
                       tr("<p><b>qIpCam</b></p>"
                          "This software uses VAPIX 3 and is adapted to Axis cameras with Firmware Version 5.x only."
                          "<p>Version qIpCam: %1"
                          "<p>Version %2: %3.%4"
                          "<p>Licence : GPL V3"
                          "<p>Author : Stephane List (slist@lilotux.net)"
                          "</p>")
                       .arg(VERSION)
#ifdef __GNUC__
                       .arg("GCC")
                       .arg(__GNUC__)
                       .arg(__GNUC_MINOR__)
#elif defined(_MSC_VER)
                       .arg("VC++")
                       .arg(_MSC_VER / 100)
                       .arg(_MSC_VER % 100,2,10,QChar('0'))
#else
                       .arg("Compiler")
                       .arg("?")
                       .arg("?")
#endif
                       );
}

void MainWindow::on_actionAbout_Qt_triggered()
{
    QApplication::aboutQt ();
}

void MainWindow::on_actionExit_triggered()
{
    close();
}

void MainWindow::slot_audio(const QByteArray&)
{

    // qDebug() << "A";
    //ui->label->setText(ui->label->text()+"A");
}

void MainWindow::slot_video(const QByteArray& Datav)
{
    //  qDebug() << "V";
    //ui->label->setText(ui->label->text()+"V");
    QByteArray Data = QByteArray::fromRawData(header, 0);
    if (vd != 0)
    {
        QSize Size;
        DecodedVideo_.clear();
        Data.append(Datav.right(Datav.size() - RTP_HEADER_LEN));

        if (vd->Decode(Data.data(), Data.size(), PIX_FMT_RGB24, Size, DecodedVideo_))
        {
            Image_ = QImage(reinterpret_cast<uchar*>(DecodedVideo_.data())
                            ,Size.width()
                            ,Size.height()
                            ,QImage::Format_RGB888).copy();

            ui->label->setPixmap(QPixmap::fromImage(Image_.scaled(ui->label->width(),ui->label->height()),Qt::AutoColor));
        }


    }
    else
    {
        qWarning("Video decoder not initialized!");
    }

    QFile f("bitstream.h264");
    if (!f.open(QIODevice::Append))
    {
        qDebug("can't write to file");
        return;
    }
    //f.write(Data.data() + RTP_HEADER_LEN, Data.size() - RTP_HEADER_LEN);
    f.write(Data.data(), Data.size());
    f.close();

}
