/***************************************************************************
 *   Copyright (C) 2009, 2010 by Stephane List                             *
 *   slist@lilotux.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef RECORDER_H
#define RECORDER_H

#include <QObject>
#include <QFile>
#include <QtGlobal>

extern "C"
{
#include "libavformat/avformat.h"
}
#include "rtspclientudp.h"

class recorder : public QObject
{
    Q_OBJECT

public:
    recorder(QObject *parent = 0);
    ~recorder();

    // Set streams info
    int set_video_format(QString &);
    int set_video_fps(quint32);
    int set_video_bit_rate(quint32);
    int set_audio_bit_rate(quint32);
    int set_audio_format(QString &);
    int set_audio_channels(quint32);

    void start(const QString & video_filename);
    void stop();
signals:
    void error(const QString & reason);
	public slots:
    void on_VideoAvailable(const QByteArray&);
    void on_AudioAvailable(const QByteArray&);
    void pause();
private:
    bool recording_;
    bool header_written_;
    QString filename_;
    QFile *pfile;
    AVOutputFormat *fmt;
    AVFormatContext *oc;
    AVCodecContext *contextv;
    AVStream *audio_st, *video_st;
    AVRational video_frame_rate_;

    qint64 ts_first_frame_video;
    int video_width, video_height;
    int has_b_frames;

    QString video_format_;
    quint32 video_fps_;
    quint32 video_bit_rate_;
    quint32 audio_bit_rate_;
    QString audio_format_;
    quint32 audio_channels_;

    quint32 audio_header_size_;

    int add_audio_stream(CodecID codec_id);
    int add_video_stream(CodecID codec_id);
    int write_header();
    int get_size_from_mp4_frame(void *buf);
    AVFrame* alloc_picture(int pix_fmt, int width, int height);
    int open_video(AVStream *st);
    int open_audio(AVStream *st);
};

#endif

