/***************************************************************************
 *   Copyright (C) 2009, 2010 by Stephane List                             *
 *   slist@lilotux.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*
    23/08/2010 Add mpeg4 and h264 parameters extraction from SDP
    24/10/2009 Add LSR computation

*/

//#define QT_NO_DEBUG_OUTPUT 1

//#define DEBUG_RTCP
//#define DEBUG_RTSP

#define VAPIX 3

#include "rtspclientudp.h"
#include "utils.h"

#define TELNET_PORT 23

//------------------------------------------------------------------------
/// Constructeur
//------------------------------------------------------------------------
RtspClientUdp::RtspClientUdp(QObject *parent)
    :	QObject(parent),
    watchdog_(this),
    tcpSocket(this),
    aSocket(this),
    acSocket(this),
    vSocket(this),
    vcSocket(this)
{
#ifdef DEBUG_RTSP
    qDebug() << "Creation Client RTSP unicast";
#endif
    // Creation de SSRC aleatoires
    qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));
    connect(&watchdog_, SIGNAL(timeout()), this, SLOT(watchdog()));
}

//------------------------------------------------------------------------
/// Destructeur
//------------------------------------------------------------------------
RtspClientUdp::~RtspClientUdp()
{
#ifdef DEBUG_RTSP
    qDebug() << "RtspClientUdp::~RtspClientUdp";
#endif
    reqSTOP();
#ifdef DEBUG_RTSP
    qDebug() << "RtspClientUdp::~RtspClientUdp ended (objet mort !!!)";
#endif
}

void RtspClientUdp::disconnect()
{
#ifdef DEBUG_RTSP
    qDebug() << "RtspClientUdp::disconnect";
#endif
    reqSTOP();
    HttpSock_.disconnect();
}


void RtspClientUdp::connectToHost(const QString & hostName, quint16 port)
{
    qDebug() << "Connect to " << hostName << ":" << port;
    serverName_ = hostName;
    serverPort_ = port;
    udp_server_audio_port = 0;
    udp_server_video_port = 0;
    alast_seq = 0;
    vlast_seq = 0;
    max_aSeq = 0;
    max_vSeq = 0;
    ncycle_aSeq = 0;
    ncycle_vSeq = 0;
    afirst = true;
    vfirst = true;
    remote_aSSRC = 0;
    remote_vSSRC = 0;
    alost = 0;
    alost_last = 0;
    frac_lost_ = 0;
    vlost = 0;
    vlost_last = 0;
    aLSR = 0;
    vLSR = 0;
    update_state(UnconnectedState);
    ServerType_ = ServerUnknow;
    CSeq = 1;
    video_format_ = "";

    sdp_mpeg4_profile_level_id_=0;
    sdp_mpeg4_video_config_="";
    sdp_h264_packetization_mode_=0;
    sdp_h264_profile_level_id_=0;
    sdp_h264_sprop_parameter_sets_="";

    vcontrol_ = "";
    audio_format_ = "";
    acontrol_ = "";
    video_fps_ = 0;
    video_bit_rate_ = 0;
    audio_bit_rate_ = 0;
    hasSession = false;
    use_credentials_ = false;
    PtzCapabilities_ = false;
    timeout_tcp_ = 0;
    connect(&tcpSocket, SIGNAL( error(QAbstractSocket::SocketError) ), this, SLOT( handleError(QAbstractSocket::SocketError )), Qt::DirectConnection);
    connect(&tcpSocket, SIGNAL( readyRead() ), this, SLOT( readBytes() ), Qt::DirectConnection);
    connect(&tcpSocket, SIGNAL( connected() ), this, SLOT( on_tcp_connected() ));
    connect(&tcpSocket, SIGNAL( disconnected() ), this, SLOT( on_tcp_disconnected() ));
    tcpSocket.connectToHost(hostName, port);
}

void RtspClientUdp::on_tcp_connected()
{
    if (tcpSocket.state() != QAbstractSocket::ConnectedState)
    {
        qWarning() << "tcpSocket.state != QAbstractSocket::ConnectedState";
        qDebug() << "tcpSocket.state =" << tcpSocket.state();
    }
    update_state(OptionsState);
    reqOPTIONS();
    abuf.clear();
    vbuf.clear();

    local_aSSRC = (quint32)qrand();
    local_vSSRC = (quint32)qrand();

    watchdog_.start(1000); // 1 seconde
    watchdog_value_ = WATCHDOG_TIMEOUT;
}

void RtspClientUdp::on_tcp_disconnected()
{
    if (state_ != Stopping)
    {
        qDebug() << "RtspClientUdp::on_tcp_disconnected";
        reqSTOP();
    }
}
//------------------------------------------------------------------------
/// Envoie une requete SETUP (video seule)
//------------------------------------------------------------------------
void RtspClientUdp::reqSETUPvideo()
{
    QString req, seq;
    qDebug() << "SETUP video";

    vSocket.bind();
    udp_client_video_port = vSocket.localPort();
    connect(&vSocket, SIGNAL(readyRead()),this, SLOT(receiveVideo()));

    vcSocket.bind(udp_client_video_port + 1);
    connect(&vcSocket, SIGNAL(readyRead()),this, SLOT(receivecVideo()));

    seq.setNum(CSeq++);
#if (VAPIX==3)
    req += "SETUP " + vcontrol_ + " RTSP/1.0\r\n";
#else
    req += "SETUP " + contentbase_ + vcontrol_ + " RTSP/1.0\r\n";
#endif
    req += "CSeq: " + seq + "\r\n";
    req += "Transport: RTP/AVP;unicast;client_port=";
    req += QString("%1").arg(udp_client_video_port) + "-" + QString("%1").arg(udp_client_video_port + 1) + "\r\n";
    if (hasSession)
    {
        req += "Session: " + session + "\r\n";
    }
    req += Authentication();
    req += "User-Agent: qipcam\r\n";
    req += "\r\n";

    qDebug() << req;
    tcpSocket.write(req.toAscii());
}

//------------------------------------------------------------------------
/// Envoie une requete SETUP (audio seul)
//------------------------------------------------------------------------
void RtspClientUdp::reqSETUPaudio()
{
    QString req, seq;
    qDebug() << "SETUP audio";

    aSocket.bind();
    udp_client_audio_port = aSocket.localPort();
    connect(&aSocket, SIGNAL(readyRead()),this, SLOT(receiveAudio()));

    acSocket.bind(udp_client_audio_port + 1);
    connect(&acSocket, SIGNAL(readyRead()),this, SLOT(receivecAudio()));

    seq.setNum(CSeq++);
#if (VAPIX==3)
    req += "SETUP " + acontrol_ + " RTSP/1.0\r\n";
#else
    req += "SETUP " + contentbase_ + acontrol_ + " RTSP/1.0\r\n";
#endif
    req += "CSeq: " + seq + "\r\n";
    req += "Transport: RTP/AVP;unicast;client_port=";
    req += QString("%1").arg(udp_client_audio_port) + "-" + QString("%1").arg(udp_client_audio_port + 1) + "\r\n";
    if (hasSession)
    {
        req += "Session: " + session + "\r\n";
    }
    req += Authentication();
    req += "User-Agent: qipcam\r\n";
    req += "\r\n";

    qDebug() << req;
    tcpSocket.write(req.toAscii());
}

void RtspClientUdp::decode_rtcp(QByteArray datagram, MediaType mt)
{
    if (datagram.size() >=8)
    {
        quint8 pt = (quint8)datagram[1];
        quint16 len = ((quint8)datagram[2]<<8) + (quint8)datagram[3];
        quint32 ssrc;
        ssrc =  ((quint8)datagram[4]) << 24;
        ssrc += ((quint8)datagram[5]) << 16;
        ssrc += ((quint8)datagram[6]) << 8;
        ssrc += (quint8)datagram[7];
#ifdef DEBUG_RTCP
        qDebug() << "decode_rtcp";
        qDebug() << "len = " << len;
        qDebug() << "Sender SSRC = " << ssrc;
#endif
        switch (pt) {
        case 200:
#ifdef DEBUG_RTCP
            qDebug() << "pt = 200 (Sender report)";
#endif
            if (datagram.size() >=28)
            {
                quint32 MSW;
                MSW =  ((quint8)datagram[8]) << 24;
                MSW += ((quint8)datagram[9]) << 16;
                MSW += ((quint8)datagram[10]) << 8;
                MSW += (quint8)datagram[11];
#ifdef DEBUG_RTCP
                qDebug() << "MSW = " << MSW;
#endif
                quint32 LSW;
                LSW =  ((quint8)datagram[12]) << 24;
                LSW += ((quint8)datagram[13]) << 16;
                LSW += ((quint8)datagram[14]) << 8;
                LSW += (quint8)datagram[15];
#ifdef DEBUG_RTCP
                qDebug() << "LSW = " << LSW;
#endif
                quint32 LSR;
                LSR =  ((quint8)datagram[10]) << 24;
                LSR += ((quint8)datagram[11]) << 16;
                LSR += ((quint8)datagram[12]) << 8;
                LSR += (quint8)datagram[13];

                switch (mt) {
                case MediaAudio:
                    aLSR = LSR;
                    if (t_aLSR.isValid())
                    {
                        t_aLSR.restart();
                    }
                    else
                    {
                        t_aLSR.start();
                    }
                    break;
						case MediaVideo:
                    vLSR = LSR;
                    if (t_vLSR.isValid())
                    {
                        t_vLSR.restart();
                    }
                    else
                    {
                        t_vLSR.start();
                    }
                    break;
						default:
                    qWarning("Wrong media type");
                }

                quint32 RTP_ts;
                RTP_ts =  ((quint8)datagram[16]) << 24;
                RTP_ts += ((quint8)datagram[17]) << 16;
                RTP_ts += ((quint8)datagram[18]) << 8;
                RTP_ts += (quint8)datagram[19];
#ifdef DEBUG_RTCP
                qDebug() << "RTP Timestamp = " << RTP_ts;
#endif
                quint32 spc;
                spc =  ((quint8)datagram[20]) << 24;
                spc += ((quint8)datagram[21]) << 16;
                spc += ((quint8)datagram[22]) << 8;
                spc += (quint8)datagram[23];
#ifdef DEBUG_RTCP
                qDebug() << "Sender's packet count = " << spc;
#endif
                quint32 soc;
                soc =  ((quint8)datagram[24]) << 24;
                soc += ((quint8)datagram[25]) << 16;
                soc += ((quint8)datagram[26]) << 8;
                soc += (quint8)datagram[27];
#ifdef DEBUG_RTCP
                qDebug() << "Sender's octet count = " << soc;
#endif
            }
            else
            {
                qWarning("Incomplete Sender Report");
            }
            if ((datagram.size() - (4+(len<<2))) > 0) {
                // Decode la suite
#ifdef DEBUG_RTCP
                qDebug() << "----\nappel reccursif de decode_rtcp";
#endif
                decode_rtcp(datagram.mid(4+(len<<2)), mt);
            }
            break;
			case 201:
#ifdef DEBUG_RTCP
            qDebug() << "pt = 201 (receiver report)";
#endif
            break;
			case 202:
#ifdef DEBUG_RTCP
            qDebug() << "pt = 202 (Source Description)";
#endif
            break;
			case 203:
            qWarning() << "pt = 203 (Bye)";
            reqSTOP();
            return;
			default:
            qWarning() << "pt = " << pt << "(Unknown RTCP packet type)";
            break;
        }
    }
    else
    {
        qWarning() << "RTCP datagram too short, size = " << datagram.size();
    }
}

//------------------------------------------------------------------------
/// Recoit du RTCP audio
//------------------------------------------------------------------------
void RtspClientUdp::receivecAudio()
{
    int i;

#ifdef DEBUG_RTCP
    qDebug("RTCP audio");
#endif
    while (acSocket.hasPendingDatagrams())
    {
        QByteArray datagram;
        datagram.resize(acSocket.pendingDatagramSize());
        QHostAddress sender;
        quint16 senderPort;
        acSocket.readDatagram(datagram.data(), datagram.size(), &sender, &senderPort);

        if (senderPort != udp_server_audio_port+1)
        {
            qDebug() << "senderPort              = " << senderPort;
            qDebug() << "udp_server_audio_port+1 = " << udp_server_audio_port+1;
            qWarning() << "Ignore RTCP packet, UDP server port wrong for RTCP audio!";
            break;
        }

        decode_rtcp(datagram, MediaAudio);

        // Envoi d'une reponse pour eviter le timeout de 60s des Axis
        datagram.resize(52);
        //RTCP Receiver Report
        datagram[0] = 0x80 | 1; // RFC 1889 v2 | 1 source
        datagram[1] =  201; // Packet Type: Receiver Report (201)
        datagram[2] = 0x00; // Length MSB
        datagram[3] = 0x07; // length LSB
        datagram[4] = static_cast<quint8>(local_aSSRC >> 24);
        datagram[5] = static_cast<quint8>(local_aSSRC >> 16);
        datagram[6] = static_cast<quint8>(local_aSSRC >> 8);
        datagram[7] = static_cast<quint8>(local_aSSRC);
#ifdef DEBUG_RTCP
        qDebug() << "local_aSSRC" << local_aSSRC;
#endif
        i = 8;

        //Source 1
        datagram[i + 0] = static_cast<quint8>(remote_aSSRC >> 24);
        datagram[i + 1] = static_cast<quint8>(remote_aSSRC >> 16);
        datagram[i + 2] = static_cast<quint8>(remote_aSSRC >> 8);
        datagram[i + 3] = static_cast<quint8>(remote_aSSRC);
#ifdef DEBUG_RTCP
        qDebug() << "remote_aSSRC" << remote_aSSRC;
#endif
        i+=4;

        //Fraction Lost
        frac_lost_ = ((alost - alost_last) << 8) / 1; // TODO

        datagram[i + 0] = qMin(alost - alost_last, static_cast<unsigned int>(0xff)); // should be multiplied by 0xFF and divided by number of sequence since last time, taking into account ncycle also..
        //Cumulative number of packets lost
        datagram[i + 1] = static_cast<quint8>(alost >> 16);
        datagram[i + 2] = static_cast<quint8>(alost >> 8);
        datagram[i + 3] = static_cast<quint8>(alost);
        alost_last = alost;
        i += 4;

        //Extended highest sequence number received
        datagram[i + 0] = static_cast<quint8>(ncycle_aSeq >> 8);
        datagram[i + 1] = static_cast<quint8>(ncycle_aSeq);
#ifdef DEBUG_RTCP
        qDebug() << "ncycle_aSeq" << ncycle_aSeq;
#endif

        //Highest sequence number received
        datagram[i + 2] = static_cast<quint8>(max_aSeq >> 8);
        datagram[i + 3] = static_cast<quint8>(max_aSeq);
#ifdef DEBUG_RTCP
        qDebug() << "max_aSeq" << max_aSeq;
#endif
        i += 4;

        //Interrarrival jitter TODO
        datagram[i + 0] = 0;
        datagram[i + 1] = 0;
        datagram[i + 2] = 0;
        datagram[i + 3] = 0;
        i += 4;

        //Last SR timestamp
        datagram[i + 0] = static_cast<quint8>(aLSR >> 24);
        datagram[i + 1] = static_cast<quint8>(aLSR >> 16);
        datagram[i + 2] = static_cast<quint8>(aLSR >> 8);
        datagram[i + 3] = static_cast<quint8>(aLSR);
        i += 4;
	
        // delay since last SR (DLSR): 32 bits
        // The delay, expressed in units of 1/65536 seconds, between receiving the last SR packet from source SSRC_n and sending this reception report block. If no SR packet has been received yet from SSRC_n, the DLSR field is set to zero.
        quint32 DLSR = t_aLSR.elapsed() * 65.536;
        datagram[i + 0] = static_cast<quint8>(DLSR >> 24);
        datagram[i + 1] = static_cast<quint8>(DLSR >> 16);
        datagram[i + 2] = static_cast<quint8>(DLSR >> 8);
        datagram[i + 3] = static_cast<quint8>(DLSR);
        i += 4;

        //RTCP Source description
        datagram[i + 0] = 0x80 | 1; // RFC 1889 v2 | 1 source
        datagram[i + 1] =  202; // Packet Type: Source description (202)
        datagram[i + 2] = 0x00; // Length MSB
        datagram[i + 3] = 0x04; // length LSB
        datagram[i + 4] = static_cast<quint8>(local_aSSRC >> 24);
        datagram[i + 5] = static_cast<quint8>(local_aSSRC >> 16);
        datagram[i + 6] = static_cast<quint8>(local_aSSRC >> 8);
        datagram[i + 7] = static_cast<quint8>(local_aSSRC);
        i += 8;

        //SDES items
        datagram[i+0] = 0x01; // Type: CNAME (user and domain)
        datagram[i+1] = 0x06; // length
        i += 2;

        datagram[i + 0] = 'p'; //0x70; // p
        datagram[i + 1] = 'o'; //0x6f; // o
        datagram[i + 2] = 's'; //0x73; // s
        datagram[i + 3] = 't'; //0x74; // t
        datagram[i + 4] = 'e'; //0x65; // e
        datagram[i + 5] = '2'; //0x32; // 2
        datagram[i + 6] = 0; // END
        datagram[i + 7] = 0; // END
        datagram[i + 8] = 0; // END
        datagram[i + 9] = 0; // END
        i += 10;

        if (i % 4 != 0)
        {
            qWarning() << "Warning, RTCP should ba a multiple of 4 ! i = " << i;
        }
        acSocket.writeDatagram(datagram.data(), datagram.size(), sender, senderPort);
    }
}

//------------------------------------------------------------------------
/// Recoit du RTCP video
//------------------------------------------------------------------------
void RtspClientUdp::receivecVideo()
{
    while (vcSocket.hasPendingDatagrams())
    {
        QByteArray datagram;
        datagram.resize(vcSocket.pendingDatagramSize());
        QHostAddress sender;
        quint16 senderPort;
        int i;

#ifdef DEBUG_RTCP
	qDebug("****\n**** Receive RTCP video\n****");
#endif
        vcSocket.readDatagram(datagram.data(), datagram.size(), &sender, &senderPort);

        if (senderPort != udp_server_video_port+1)
        {
            qDebug() << "senderPort              = " << senderPort;
            qDebug() << "udp_server_video_port+1 = " << udp_server_video_port+1;
            qWarning() << "Ignore RTCP packet, UDP server port wrong for RTCP video!";
            break;
        }

        decode_rtcp(datagram,  MediaVideo);

        // Envoi d'une reponse pour eviter le timeout de 60s des Axis
        datagram.resize(52);
        //RTCP Receiver Report

        // version (V): 2 bits
        // Identifies the version of RTP, which is the same in RTCP packets as in RTP data packets. The version defined by this specification is two (2).
        quint8 V = 2;

        // padding (P): 1 bit
        // If the padding bit is set, this RTCP packet contains some additional padding octets at the end which are not part of the control information. The last octet of the padding is a count of how many padding octets should be ignored. Padding may be needed by some encryption algorithms with fixed block sizes. In a compound RTCP packet, padding should only be required on the last individual packet because the compound packet is encrypted as a whole.
        quint8 P = 0;

        // reception report count (RC): 5 bits
        // The number of reception report blocks contained in this packet. A value of zero is valid.
        quint8 RC = 1;
        datagram[0] = (V<<6) | (P<<5) | (RC); // RFC 1889 v2 | 0 padding | 1 source
        datagram[1] =  201; // Packet Type: Receiver Report (201)
        datagram[2] = 0x00; // Length MSB
        datagram[3] = 0x07; // length LSB
        datagram[4] = static_cast<quint8>(local_vSSRC >> 24);
        datagram[5] = static_cast<quint8>(local_vSSRC >> 16);
        datagram[6] = static_cast<quint8>(local_vSSRC >> 8);
        datagram[7] = static_cast<quint8>(local_vSSRC);
#ifdef DEBUG_RTCP
        qDebug() << "local_vSSRC" << local_vSSRC;
#endif
        i = 8;

        //Source 1
        datagram[i + 0] = static_cast<quint8>(remote_vSSRC >> 24);
        datagram[i + 1] = static_cast<quint8>(remote_vSSRC >> 16);
        datagram[i + 2] = static_cast<quint8>(remote_vSSRC >> 8);
        datagram[i + 3] = static_cast<quint8>(remote_vSSRC);
#ifdef DEBUG_RTCP
        qDebug() << "remote_vSSRC" << remote_vSSRC;
#endif
        i += 4;

        //Fraction Lost
        datagram[i + 0] = qMin(vlost - vlost_last, static_cast<unsigned int>(0xff)); // should be multiplied by 0xFF and divided by number of sequence since last time, taking into account ncycle also..
        //Cumulative number of packets lost
        datagram[i + 1] = static_cast<quint8>(vlost >> 16);
        datagram[i + 2] = static_cast<quint8>(vlost >> 8);
        datagram[i + 3] = static_cast<quint8>(vlost);
        vlost_last = vlost;
        i += 4;

        //Extended highest sequence number received
        datagram[i + 0] = static_cast<quint8>(ncycle_vSeq >> 8);
        datagram[i + 1] = static_cast<quint8>(ncycle_vSeq);
#ifdef DEBUG_RTCP
        qDebug() << "ncycle_vSeq" << ncycle_vSeq;
#endif

        //Highest sequence number received
        datagram[i + 2] = static_cast<quint8>(max_vSeq >> 8);
        datagram[i + 3] = static_cast<quint8>(max_vSeq);
#ifdef DEBUG_RTCP
        qDebug() << "max_vSeq" << max_vSeq;
#endif
        i += 4;

        //Interrarrival jitter TODO
        datagram[i + 0] = 0;
        datagram[i + 1] = 0;
        datagram[i + 2] = 0;
        datagram[i + 3] = 0;
        i += 4;

        //Last SR timestamp
        datagram[i + 0] = static_cast<quint8>(vLSR >> 24);
        datagram[i + 1] = static_cast<quint8>(vLSR >> 16);
        datagram[i + 2] = static_cast<quint8>(vLSR >> 8);
        datagram[i + 3] = static_cast<quint8>(vLSR);
        i += 4;

        // delay since last SR (DLSR): 32 bits
        // The delay, expressed in units of 1/65536 seconds, between receiving the last SR packet from source SSRC_n and sending this reception report block. If no SR packet has been received yet from SSRC_n, the DLSR field is set to zero.
        quint32 DLSR = t_vLSR.elapsed() * 65.536;
        datagram[i + 0] = static_cast<quint8>(DLSR >> 24);
        datagram[i + 1] = static_cast<quint8>(DLSR >> 16);
        datagram[i + 2] = static_cast<quint8>(DLSR >> 8);
        datagram[i + 3] = static_cast<quint8>(DLSR);
        i += 4;

        //RTCP Source description
        datagram[i + 0] = 0x80 | 1; // RFC 1889 v2 | 1 source
        datagram[i + 1] =  202; // Packet Type: Source description (202)
        datagram[i + 2] = 0x00; // Length MSB
        datagram[i + 3] = 0x04; // length LSB
        datagram[i + 4] = static_cast<quint8>(local_vSSRC >> 24);
        datagram[i + 5] = static_cast<quint8>(local_vSSRC >> 16);
        datagram[i + 6] = static_cast<quint8>(local_vSSRC >> 8);
        datagram[i + 7] = static_cast<quint8>(local_vSSRC);
        i += 8;

        //SDES items
        datagram[i + 0] = 0x01; // Type: CNAME (user and domain)
        datagram[i + 1] = 0x06; // length
        i += 2;

        datagram[i + 0] = 'p'; // 0x70; // p
        datagram[i + 1] = 'o'; // 0x6f; // o
        datagram[i + 2] = 's'; // 0x73; // s
        datagram[i + 3] = 't'; // 0x74; // t
        datagram[i + 4] = 'e'; // 0x65; // e
        datagram[i + 5] = '2'; // 0x32; // 2
        datagram[i + 6] = 0; // END
        datagram[i + 7] = 0; // END
        datagram[i + 8] = 0; // END
        datagram[i + 9] = 0; // END
        i += 10;

        if (i % 4 != 0)
        {
            qWarning() << "Warning, RTCP should be a multiple of 4 ! i = " << i;
        }
#ifdef DEBUG_RTCP
        qDebug() << "Send answer to RTCP to avoid the timeout of 60s with Axis camera";
        qDebug() << "****\n****Decodage de ce qu'on envoie ...\n****";
        decode_rtcp(datagram, MediaVideo);
#endif
        vcSocket.writeDatagram(datagram.data(), datagram.size(), sender, senderPort);
    }
}

//------------------------------------------------------------------------
/// Recoit des donnees video
//
// Recompose 1 trame RTP contenant une trame video entiere avant d'envoyer un signal
//
// Attention, la taille dans l'en tete RTP est mauvaise dans la trame
// reconstruite, il faudra utiliser la taille du QByteArray !
//
//------------------------------------------------------------------------
void RtspClientUdp::receiveVideo()
{
    quint16 seq;
    quint32 ts;
    QString q;
    quint8 payload;

    while (vSocket.hasPendingDatagrams()) {
        QByteArray datagram;
        datagram.resize(vSocket.pendingDatagramSize());
        QHostAddress sender;
        quint16 senderPort;
        vSocket.readDatagram(datagram.data(), datagram.size(), &sender, &senderPort);

        if (senderPort != udp_server_video_port)
        {
            qWarning("video coming from bad udp port, ignore frame");
            qDebug() << "senderPort : " << senderPort << " udp_server_video_port : " << udp_server_video_port;
            break;
        }

        if (datagram.size() <= RTP_HEADER_LEN )
            break;
        payload = static_cast<quint8>(datagram[1]);

        seq = (static_cast<quint8>(datagram[2]) << 8) + static_cast<quint8>(datagram[3]);
        ts  = (static_cast<quint8>(datagram[4]) << 24);
        ts += (static_cast<quint8>(datagram[5]) << 16);
        ts += (static_cast<quint8>(datagram[6]) << 8);
        ts += (static_cast<quint8>(datagram[7]) << 0);

        if (vfirst)
        {
            vlast_seq = seq;
            max_vSeq = seq;
        }

        // Detection du rebouclage de seq
        if ((seq < 0x10) && (max_vSeq > 0xffff - 0x10))
        {
            ncycle_vSeq++;
            max_vSeq = seq;
            vlast_seq = seq;
        }
        else
        {
            max_vSeq = qMax(max_vSeq, seq);
        }

        if (! vbuf.isEmpty())
        {
            datagram = datagram.right(datagram.size() - RTP_HEADER_LEN);
        }

        if ((seq == vlast_seq + 1) || (vfirst))
        {
            vfirst = false;
            vlast_seq = seq;
            vbuf.append(datagram);
            if ((payload & MARKER_BIT) != 0)
            {
                emit(VideoAvailable(vbuf));
                vbuf.clear();
            }
        }
        else if (seq > vlast_seq + 1)
        {
            vbuf.clear();
            vlost += seq - vlast_seq - 1;
            qDebug() << "Lost" << seq - vlast_seq - 1 << "frame(s)";
            vlast_seq = seq;
            if ((payload & MARKER_BIT) != 0)
            {
                emit(VideoAvailable(datagram));
            }
            else
            {
                vbuf.append(datagram);
            }
        }
        else
        {
            qDebug() << "Ignore frame (seq =" << seq << " vlast_seq =" << vlast_seq << ")";
        }
    }
}


//------------------------------------------------------------------------
/// Recoit des donnees audio
//
// Recompose 1 trame RTP contenant une trame audio entiere avant d'envoyer un signal
//
// Attention, la taille dans l'en tete RTP est mauvaise dans la trame
// reconstruite, il faudra utiliser la taille du QByteArray !
//
//------------------------------------------------------------------------
void RtspClientUdp::receiveAudio()
{
    quint16 seq;
    quint32 ts;
    QString q;
    quint8 payload;

    while (aSocket.hasPendingDatagrams()) {
        QByteArray datagram;
        datagram.resize(aSocket.pendingDatagramSize());
        QHostAddress sender;
        quint16 senderPort;
        aSocket.readDatagram(datagram.data(), datagram.size(), &sender, &senderPort);

        if (senderPort != udp_server_audio_port)
        {
            qWarning("audio coming from bad udp port, ignore frame");
            qDebug() << "senderPort : " << senderPort << " udp_server_audio_port : " << udp_server_audio_port;
            break;
        }

        if (datagram.size() <= RTP_HEADER_LEN )
            break;
        payload = static_cast<quint8>(datagram[1]);

        seq = (static_cast<quint8>(datagram[2]) << 8) + static_cast<quint8>(datagram[3]);
        ts  = (static_cast<quint8>(datagram[4]) << 24);
        ts += (static_cast<quint8>(datagram[5]) << 16);
        ts += (static_cast<quint8>(datagram[6]) << 8);
        ts += (static_cast<quint8>(datagram[7]) << 0);

        if (afirst)
        {
            alast_seq = seq;
            max_aSeq = seq;
        }

        // Detection du rebouclage de seq
        if ((seq < 0x10) && (max_aSeq > 0xffff - 0x10))
        {
            ncycle_aSeq++;
            max_aSeq = seq;
            alast_seq = seq;
        }
        else
        {
            max_aSeq = qMax(max_aSeq, seq);
        }

        if (!abuf.isEmpty())
        {
            datagram = datagram.right(datagram.size()-RTP_HEADER_LEN);
        }

        if ((seq == alast_seq + 1) || (afirst))
        {
            afirst = false;
            alast_seq = seq;
            abuf.append(datagram);
            if ((payload & MARKER_BIT) != 0)
            {
                emit(AudioAvailable(abuf));
                abuf.clear();
            }
        }
        else if (seq > alast_seq + 1)
        {
            abuf.clear();
            alost += seq - alast_seq - 1;
            alast_seq = seq;
            if ((payload & MARKER_BIT) != 0)
            {
                emit(AudioAvailable(datagram));
            }
            else
            {
                abuf.append(datagram);
            }
        }
        else
        {
            qDebug() << "Ignore frame (seq =" << seq << " alast_seq =" << alast_seq << ")";
        }
    }
}

//------------------------------------------------------------------------
/// Affiche une erreur (slot)
//------------------------------------------------------------------------
void RtspClientUdp::handleError(QAbstractSocket::SocketError socketError)
{
    switch (socketError)
    {
    case QAbstractSocket::RemoteHostClosedError:
        emit error("RemoteHostClosedError");
        break;
    case QAbstractSocket::HostNotFoundError:
        emit error("The host was not found. Please check the host name and port settings.");
        break;
    case QAbstractSocket::ConnectionRefusedError:
        emit error("The connection was refused by the peer. Make sure the server is running, and check that the host name and port settings are correct.");
        break;
    default:
        emit error("The following error occurred: " + tcpSocket.errorString());
    }
    reqSTOP();
}

//------------------------------------------------------------------------
/// Envoie la requete OPTIONS
//------------------------------------------------------------------------
void RtspClientUdp::reqOPTIONS()
{
    QString req;

    qDebug() << "OPTIONS";
#if (VAPIX==3)
    req += "OPTIONS * RTSP/1.0\r\n";
#else
#if (VAPIX < 3)
    req += "OPTIONS rtsp://" + serverName_ + ":" + QString::number(serverPort_) + " RTSP/1.0\r\n";
#endif
#error "VAPIX version not supported"
#endif
    req += "CSeq: " + QString::number(CSeq++) + "\r\n";
    req += "User-Agent: qipcam\r\n";
    req += "\r\n";

    qDebug() << req;
    tcpSocket.write(req.toAscii());
}

void RtspClientUdp::reqDESCRIBE()
{
    QString req, seq;
    seq.setNum(CSeq++);

    qDebug() << "DECRIBE";

#if (VAPIX==3)
        req += "DESCRIBE rtsp://" + serverName_ + "/axis-media/media.amp?videocodec=h264&resolution=640x480 RTSP/1.0\r\n";
        //req += "DESCRIBE rtsp://" + serverName_ + "/axis-media/media.amp?videocodec=mpeg4&resolution=640x480 RTSP/1.0\r\n";
#else
#if (VAPIX==2)
        req += "DESCRIBE rtsp://" + serverName_ + ":" + QString::number(serverPort_) +"/mpeg4/1/media.amp RTSP/1.0\r\n";
#endif
#endif
    req += "CSeq: " + seq + "\r\n";
    req += "Accept: application/sdp\r\n";
    req += Authentication();
    req += "\r\n";

    qDebug() << req;
    tcpSocket.write(req.toAscii());
}

void RtspClientUdp::reqPLAYaudio()
{
    QString req, seq;

    qDebug() << "PLAY audio";
    seq.setNum(CSeq++);
#if (VAPIX==3)
    req =  "PLAY " + acontrol_ + " RTSP/1.0\r\n";
#else
    req =  "PLAY " + contentbase_ + acontrol_ + " RTSP/1.0\r\n";
#endif
    req += "CSeq: " + seq + "\r\n";
    req += "Session: " + session + "\r\n";
    req += "Range: npt=0.000-\r\n";
    req += Authentication();
    req += "\r\n";
    qDebug() << req;
    tcpSocket.write(req.toAscii());
}

void RtspClientUdp::reqPLAYvideo()
{
    QString req;

    qDebug() << "PLAY video";
#if (VAPIX==3)
    req =  "PLAY " + vcontrol_ + " RTSP/1.0\r\n";
#else
    req =  "PLAY " + contentbase_ + vcontrol_ + " RTSP/1.0\r\n";
#endif
    req += "CSeq: " + QString::number(CSeq++) + "\r\n";
    req += "Session: " + session + "\r\n";
    req += "Range: npt=0.000-\r\n";
    req += Authentication();
    req += "\r\n";
    qDebug() << req;
    tcpSocket.write(req.toAscii());
}

//------------------------------------------------------------------------
/// Arrete le flux
//------------------------------------------------------------------------
void RtspClientUdp::reqSTOP()
{
    QString req, seq;
    seq.setNum(CSeq++);

    if (state_ == Stopping)
        return;
    update_state(Stopping);

    if (tcpSocket.state() == QAbstractSocket::ConnectedState)
    {
        qDebug() << "TEARDOWN";
        tcpSocket.disconnect(); // disconnect SIGNALS to avoid a lot of code beeing called when Axis closes the socket
#if (VAPIX==3)
        req =  "TEARDOWN " + vcontrol_ +" RTSP/1.0\r\n";
#else
        req =  "TEARDOWN " + contentbase_ + vcontrol_ +" RTSP/1.0\r\n";
#endif
        req += "CSeq: " + seq + "\r\n";
        req += "Session: " + session + "\r\n";
        req += "User-Agent: qipcam\r\n";
        req += "\r\n";
        qDebug() << req;
        tcpSocket.write(req.toAscii());
        tcpSocket.flush();

        if (!acontrol_.isEmpty())
        {
            seq.setNum(CSeq++);
#if (VAPIX==3)
            req =  "TEARDOWN " + acontrol_ + " RTSP/1.0\r\n";
#else
            req =  "TEARDOWN " + contentbase_ + acontrol_ + " RTSP/1.0\r\n";
#endif
            req += "CSeq: " + seq + "\r\n";
            req += "Session: " + session + "\r\n";
            req += "User-Agent: qipcam\r\n";
            req += "\r\n";
            qDebug() << req;
            tcpSocket.write(req.toAscii());
            tcpSocket.flush();
        }
    }
    else
    {
        qDebug() << "TCP socket not connected, don't send TEARDOWN";
        qDebug() << "TCP socket state" << tcpSocket.state();
    }

    tcpSocket.close();

    aSocket.disconnect();
    aSocket.close();

    vSocket.disconnect();
    vSocket.close();

    acSocket.disconnect();
    acSocket.close();

    vcSocket.disconnect();
    vcSocket.close();

    emit disconnected();

    watchdog_.stop();
}

//------------------------------------------------------------------------
/// Lit des octets recus (slot)
//------------------------------------------------------------------------
void RtspClientUdp::readBytes()
{
    if (state_ == ReceivingData)
    {
        QByteArray buf;
        receiveDATA(buf);
    }
    else
    {
        static char buf[3000] = {0};
        static int i = 0;

        while ((tcpSocket.bytesAvailable() > 0) && (state_ != ReceivingSDP))
        {
            qint64 lineLength = tcpSocket.readLine(buf + i, sizeof buf -i);
            if ((buf[i] == '\r') && (buf[i+1] == '\n'))
            {
                qDebug() << "RECV" <<buf;
                QByteArray answer(buf);
                decode_rtsp(answer);
                i = 0;
                memset(buf, 0, sizeof(buf));
            }
            else
            {
                i+=lineLength;
            }
        }
        if (state_ == ReceivingSDP)
        {
            receiveSDP();
        }
    }
}

//------------------------------------------------------------------------
/// Lit une reponse SDP
//------------------------------------------------------------------------
void RtspClientUdp::receiveSDP()
{
    char buf[3000] = {0};

    while ((tcpSocket.bytesAvailable() > 0) && (state_ == ReceivingSDP) && (SDPlen != SDP_string.size()))
    {
        qint64 len = tcpSocket.read(buf, SDPlen - SDP_string.size());
        if (len >= 0)
        {
            SDP_string += buf;
            memset(buf, 0, sizeof(buf));
        }
        else
        {
            emit error("Error while receiving SDP");
            reqSTOP();
        }
    }
    if (SDP_string.size() == SDPlen)
    {
        parseSDP();
        update_state(SetupVideo);
        reqSETUPvideo();
    }
    if (state_ != ReceivingSDP)
    {
        readBytes();
    }
}
/*

SDP_video en MPEG4:
 " 0 RTP/AVP 96
b=AS:8000
a=framerate:10.0
a=control:rtsp://192.168.1.72/axis-media/media.amp/trackID=1?videocodec=mpeg4&resolution=640x480
a=rtpmap:96 MP4V-ES/90000
a=fmtp:96 profile-level-id=245; config=000001B0F5000001B509000001000000012000C88D881F4514043C14103F



SDP_video en H264:
 " 0 RTP/AVP 96
b=AS:50000
a=framerate:10.0
a=control:rtsp://192.168.1.72/axis-media/media.amp/trackID=1?videocodec=h264&resolution=640x480
a=rtpmap:96 H264/90000
a=fmtp:96 packetization-mode=1; profile-level-id=420029; sprop-parameter-sets=Z0IAKeNQFAe2AtwEBAaQeJEV,aM48gA==
"
*/
void RtspClientUdp::parseSDP_video()
{
    int i;
    QString tmp;
    QString chaine_debut;
    QString chaine_fin;

    qDebug() << "Recherche port UDP video dans le SDP";
    tmp = SDP_video.simplified();
    chaine_fin = " ";
    i = tmp.indexOf(chaine_fin);
    tmp.truncate(i);
    udp_server_video_port = tmp.toInt();
    qDebug() << "udp_server_video_port=" << udp_server_video_port;

    qDebug() << "Recherche control video dans SDP";
    chaine_debut = "a=control:";
    chaine_fin   = "\n";
    i = SDP_video.indexOf(chaine_debut) + chaine_debut.size();
    vcontrol_ = SDP_video.mid(i);
    i = vcontrol_.indexOf(chaine_fin);
    vcontrol_.truncate(i);
    vcontrol_ = vcontrol_.simplified(); /* we can find a \r before \n */
    qDebug() << "vcontrol_:" << vcontrol_;

    qDebug() << "Recherche fps dans SDP video";
    chaine_debut = "a=framerate:";
    chaine_fin   = ".";
    i = SDP_video.indexOf(chaine_debut);
    tmp = SDP_video.mid(i+chaine_debut.size()).simplified();
    i = tmp.indexOf(chaine_fin);
    tmp.resize(i);
    video_fps_ = tmp.toInt();
    qDebug() << "fps:" << video_fps_;

    // TODO search bitrate
    video_bit_rate_ = 90000;

    qDebug() << "Search video format in SDP video";
    chaine_debut = "a=rtpmap:96";
    chaine_fin   = "/90000";
    i = SDP_video.indexOf(chaine_debut) + chaine_debut.size();
    video_format_ = SDP_video.mid(i).simplified();
    i = video_format_.indexOf(chaine_fin);
    video_format_.truncate(i);
    qDebug() << "Video Format:" << video_format_; // MP4V-ES, MPEG4, H264 or NONE

    if (video_format_ == "MP4V-ES")
    {
        parseSDP_video_mpeg4();
    }
    else if (video_format_ == "H264")
    {
        parseSDP_video_h264();
    }
}

/* a=fmtp:96 profile-level-id=245; config=000001B0F5000001B509000001000000012000C88D881F4514043C14103F */
void RtspClientUdp::parseSDP_video_mpeg4()
{
    int i;
    QString tmp;
    QString chaine_debut;
    QString chaine_fin;

    qDebug() << "Recherche profile-level-id dans SDP video";
    chaine_debut = "profile-level-id=";
    chaine_fin = ";";
    i = SDP_video.indexOf(chaine_debut) + chaine_debut.size();
    tmp = SDP_video.mid(i).simplified();
    i = tmp.indexOf(chaine_fin);
    tmp.truncate(i);
    sdp_mpeg4_profile_level_id_ = tmp.toInt();
    qDebug() << "sdp_mpeg4_profile_level_id_ = " << sdp_mpeg4_profile_level_id_;

    qDebug() << "Recherche config dans SDP video";
    chaine_debut = "config=";
    i = SDP_video.indexOf(chaine_debut);
    sdp_mpeg4_video_config_ = SDP_video.mid(i+chaine_debut.size()).simplified();
    qDebug() << "sdp_mpeg4_video_config_ = " << sdp_mpeg4_video_config_;
}

/* a=fmtp:96 packetization-mode=1; profile-level-id=420029; sprop-parameter-sets=Z0IAKeNQFAe2AtwEBAaQeJEV,aM48gA== */
/* See http://tools.ietf.org/html/rfc3984 */
void RtspClientUdp::parseSDP_video_h264()
{
    int i;
    QString tmp;
    QString chaine_debut;
    QString chaine_fin;

    qDebug() << "Recherche packetization-mode dans SDP video";
    chaine_debut = "packetization-mode=";
    chaine_fin = ";";
    i = SDP_video.indexOf(chaine_debut) + chaine_debut.size();
    tmp = SDP_video.mid(i).simplified();
    i = tmp.indexOf(chaine_fin);
    tmp.truncate(i);
    sdp_h264_packetization_mode_ = tmp.toInt();
    qDebug() << "sdp_h264_packetization_mode_" << sdp_h264_packetization_mode_;


    qDebug() << "Recherche profile-level-id dans SDP video";
    chaine_debut = "profile-level-id=";
    chaine_fin = ";";
    i = SDP_video.indexOf(chaine_debut) + chaine_debut.size();
    tmp = SDP_video.mid(i).simplified();
    i = tmp.indexOf(chaine_fin);
    tmp.truncate(i);
    sdp_h264_profile_level_id_ = tmp.toInt();
    qDebug() << "sdp_h264_profile_level_id_" << sdp_h264_profile_level_id_;


    qDebug() << "Recherche sprop-parameter-sets dans SDP video";
    chaine_debut = "sprop-parameter-sets=";
    i = SDP_video.indexOf(chaine_debut) + chaine_debut.size();
    sdp_h264_sprop_parameter_sets_ = SDP_video.mid(i).simplified();
    qDebug() << "sdp_h264_sprop_parameter_sets_" << sdp_h264_sprop_parameter_sets_;
}


void RtspClientUdp::parseSDP_audio()
{
    int i;
    QString tmp;
    QString chaine_debut;
    QString chaine_fin;

    qDebug() << "Recherche port UDP audio dans le SDP";
    tmp = SDP_audio.simplified();
    chaine_fin = " ";
    i = tmp.indexOf(chaine_fin);
    tmp.truncate(i);
    udp_server_audio_port = tmp.toInt();

    qDebug() << "Recherche control audio dans SDP";
    chaine_debut = "a=control:";
    i = SDP_audio.indexOf(chaine_debut) + chaine_debut.size();
    acontrol_ = SDP_audio.mid(i);
    chaine_fin = "\n";
    i = acontrol_.indexOf(chaine_fin);
    acontrol_.truncate(i);
    acontrol_ = acontrol_.simplified(); /* we can find a \r before \n, so remove it */

    QRegExp freqCap("a=rtpmap:\\d+.*/(\\d+).*\n");
    //FIXME plusieurs protocoles audio (plusieurs baudrate)
    if (freqCap.indexIn(SDP_audio) > -1)
    {
        audio_bit_rate_ = freqCap.cap(1).toUInt();
        qDebug() << "baudrate : " << audio_bit_rate_;
    }
    else
    {
        qWarning() << "baudrate not found: assuming 8000";
        audio_bit_rate_ = 8000;
    }


    qDebug() << "Recherche format dans SDP audio";
    if (SDP_audio.contains("mpeg4-generic"))
        audio_format_ = "AAC";
    else if (SDP_audio.contains("RTP/AVP 8"))
        audio_format_ = "ALAW";
    else if (SDP_audio.contains("G726"))
        audio_format_ = "G726";
    else if (SDP_audio.contains("audio/MPA"))
        audio_format_ = "MP3";
    else if (SDP_audio.contains("RTP/AVP 0"))
        audio_format_ = "ULAW";
    else if (SDP_audio.contains("SPEEX"))
        audio_format_ = "SPEEX";
    else
        audio_format_ = "NONE";
}

void RtspClientUdp::parseSDP()
{
    int i;
    QString chaine_debut;
    QString chaine_fin;

    qDebug() << "SDP_string" << SDP_string;

    if (SDP_string.contains("m=video"))
    {
        chaine_debut = "m=video";
        i = SDP_string.indexOf(chaine_debut) + chaine_debut.size();
        SDP_video = SDP_string.mid(i);
        chaine_fin = "m=";
        i = SDP_video.indexOf(chaine_fin);
        if (i>0)
        {
            // s'il y a un autre media, il ne fait pas parti de ce media
            SDP_video.truncate(i);
        }
        qDebug() << "SDP_video :\n" << SDP_video;
        parseSDP_video();
    }
    else
    {
        SDP_video ="";
        video_format_ = "NONE";
    }

    if (SDP_string.contains("m=audio"))
    {
        audio_channels_ = 1; // toutes les cameras sont mono
        chaine_debut = "m=audio";
        i = SDP_string.indexOf(chaine_debut);
        i += chaine_debut.size();
        SDP_audio = SDP_string.mid(i);
        chaine_debut = "m=";
        i = SDP_audio.indexOf(chaine_debut);
        if (i>0)
        {
            // s'il y a un autre media, il ne fait pas parti de ce media
            SDP_audio.truncate(i);
        }
        qDebug() << "SDP_audio :\n" << SDP_audio;
        parseSDP_audio();
    }
    else
    {
        audio_format_ = "NONE";
        audio_channels_ = 0;
        SDP_audio = "";
    }
    qDebug() << "SDP_string decoding finished";
}

//------------------------------------------------------------------------
/// Recoit des donnees
//------------------------------------------------------------------------
void RtspClientUdp::receiveDATA(QByteArray buf)
{
    buf.append(tcpSocket.readAll());
    qDebug() << "Ignoring data received on TCP :\r\n" << buf;
}

void RtspClientUdp::play()
{
    if (state_ == WaitPlay)
    {
        update_state(PlayingVideo);
        reqPLAYvideo();
    }
    else
    {
        qDebug() << "Wrong moment to call play, call ignored !";
    }
}

//------------------------------------------------------------------------
/// Decode un buffer (reponse a une requete)
//------------------------------------------------------------------------
void RtspClientUdp::decode_rtsp(QByteArray answer)
{
    switch (state_)
    {
    case OptionsState:
        if (!answer.contains("200 OK"))
        {
            qWarning("Ce serveur RTSP ne supporte pas la fonction OPTIONS !");
            emit error("Ce serveur RTSP ne supporte pas la fonction OPTIONS !");
            return;
        }
        if (!answer.contains("DESCRIBE"))
        {
            qWarning("Ce serveur RTSP ne renvoie rien dans OPTIONS !");
        }
            CheckPtzCapabilities();
        update_state(DescribeState);
        reqDESCRIBE();
        break;
		case ConfigureCplState:
        update_state(DescribeState);
        reqDESCRIBE();
        break;
		case DescribeState:
		case DescribeCredentialState:
        if (answer.contains("200 OK"))
        {
            QString s = "Content-Length:";
            int i = answer.indexOf(s);
            QString len = answer.mid(i+s.size()).simplified();
            SDPlen = len.toInt();

            qDebug() << "SDPlen=" << SDPlen;

            s = "Content-Base:";
            i = answer.indexOf(s);
            if (i > 0)
            {
                contentbase_ = answer.mid(i+s.size()).simplified();
                s = " ";
                i = contentbase_.indexOf(s);
                contentbase_.truncate(i);
            }
            else
            {
                contentbase_ = "rtsp://" + serverName_ + ":" + QString::number(serverPort_) +"/";
            }
            qDebug() << "Content-Base=" << contentbase_;

            update_state(ReceivingSDP);
            Q_ASSERT(SDPlen != 0);
        }
        else
        {
            if (answer.contains("401 Unauthorized"))
            {
                if (! use_credentials_)
                {
                    use_credentials_ = true;
                    if (! Authentication().isEmpty())
                    {
                        update_state(DescribeCredentialState);
                        qDebug() << "Trying again with credentials";
                        reqDESCRIBE();
                        break;
                    }
                    else
                    {
                        emit error("401 Unauthorized and no credentials provided");
                    }
                }
                else
                {
                    emit error("401 Unauthorized");
                }
            }
            reqSTOP();
        }
        break;
		case SetupVideo:
        if (answer.contains("200 OK"))
        {
            QString s = "Session:";
            int i = answer.indexOf(s);
            session = answer.mid(i+s.size()).simplified();
            s=";";
            i = session.indexOf(s);
            session.resize(i);
            hasSession = true;

            if (udp_server_video_port == 0)
            {
                QString chaine_debut = "server_port=";
                QString chaine_fin   = "-";
                QString tmp;
                qDebug() << "Recherche port UDP video dans reponse au setup";
                i = answer.indexOf(chaine_debut);
                tmp = answer.mid(i+chaine_debut.size()).simplified();
                i = tmp.indexOf(chaine_fin);
                tmp.resize(i);
                udp_server_video_port = tmp.toInt();
                qDebug() << "udp_server_video_port =" << udp_server_video_port;
            }
            if (remote_vSSRC == 0)
            {
                QString chaine_debut = "ssrc=";
                QString chaine_fin   = ";";
                QString tmp;
                bool ok;
                i = answer.indexOf(chaine_debut);
                tmp = answer.mid(i+chaine_debut.size()).simplified();
                i = tmp.indexOf(chaine_fin);
                tmp.resize(i);
                remote_vSSRC = tmp.toInt(&ok, 16);
                qDebug() << "remote_vSSRC =" << remote_vSSRC;
            }

            if (timeout_tcp_ == 0)
            {
                QString chaine_debut = "timeout=";
                QString chaine_fin   = "\r";
                QString tmp;
                i = answer.indexOf(chaine_debut);
                tmp = answer.mid(i+chaine_debut.size());
                i = tmp.indexOf(chaine_fin);
                tmp.resize(i);
                timeout_tcp_ = tmp.toInt();
                qDebug() << "timeout_tcp_ = " << timeout_tcp_ << " s";

                // TODO : envoyer OPTIONS toutes les timeout_tcp_ / 2 !!!
            }

            if (acontrol_.isEmpty())
            {
                qDebug("Camera without audio !");
                update_state(WaitPlay);
                emit connected();
            }
            else
            {
                update_state(SetupAudio);
                reqSETUPaudio();
            }
        }
        else
        {
            qWarning("Erreur de SETUP VIDEO");
            reqSTOP();
        }
        break;
		case SetupAudio:
        if (answer.contains("200 OK"))
        {
            qDebug() << "Recherche num. de session dans reponse au setup";
            QString s = "Session:";
            int i = answer.indexOf(s);
            session = answer.mid(i+s.size()).simplified();
            s=";";
            i = session.indexOf(s);
            session.resize(i);
            qDebug() << "session" << session;
            hasSession = true;

            if (udp_server_audio_port == 0)
            {
                QString chaine_debut = "server_port=";
                QString chaine_fin   = "-";
                QString tmp;
                i = answer.indexOf(chaine_debut);
                tmp = answer.mid(i+chaine_debut.size()).simplified();
                i = tmp.indexOf(chaine_fin);
                tmp.resize(i);
                udp_server_audio_port = tmp.toInt();
                qDebug() << "udp_server_audio_port =" << udp_server_audio_port;
            }
            if (remote_aSSRC == 0)
            {
                QString chaine_debut = "ssrc=";
                QString chaine_fin   = ";";
                QString tmp;
                bool ok;
                i = answer.indexOf(chaine_debut);
                tmp = answer.mid(i+chaine_debut.size()).simplified();
                i = tmp.indexOf(chaine_fin);
                tmp.resize(i);
                remote_aSSRC = tmp.toInt(&ok, 16);
                qDebug() << "remote_aSSRC =" << remote_aSSRC;
            }
            update_state(WaitPlay);
            emit connected();
        }
        else
        {
            qWarning("Erreur de SETUP AUDIO");
            reqSTOP();
        }
        break;
		case PlayingVideo:
        if (answer.contains("200 OK"))
        {
            if (!acontrol_.isEmpty())
            {
                update_state(PlayingAudio);
                reqPLAYaudio();
            }
            else
            {
                update_state(ReceivingData);
            }
        }
        else
        {
            qDebug("Error in PLAY video !");
            reqSTOP();
        }
        break;
		case PlayingAudio:
		case Playing:
        if (answer.contains("200 OK"))
        {
            update_state(ReceivingData);
        }
        else
        {
            qDebug("Error in PLAY audio !");
            reqSTOP();
        }
        break;
		case Stopping:
		case WaitPlay:
		case ReceivingData:
        break;
		default:
        qDebug() << "State =" << state_;
        qFatal("Cas non prevu !");
    }
}


QString & RtspClientUdp::get_video_format()
{
    return video_format_;
}

quint32 RtspClientUdp::get_video_fps()
{
    if (video_fps_ != 0)
    {
        qDebug() << "Video fps :" << video_fps_;
        return video_fps_;
    }
    else
    {
        qDebug() << "Unknown fps, using 30 as default value";
        return 30; /* default fps */
    }
}

quint32 RtspClientUdp::get_video_bit_rate()
{
    if (video_bit_rate_ != 0)
    {
        qDebug() << "Video bit rate :" << video_bit_rate_;
        return video_bit_rate_;
    }
    else
    {
        qDebug() << "Unknown video bit rate, using 250000 as default value";
        return 250000;
    }
}

quint32 RtspClientUdp::get_audio_bit_rate()
{
    if (audio_bit_rate_ != 0)
    {
        qDebug() << "Audio bit rate :" << audio_bit_rate_;
        return audio_bit_rate_;
    }
    else
    {
        if ((audio_format_ == "") || (audio_format_ == "NONE"))
        {
            return 0;
        }
        else
        {
            qDebug() << "Unknown audio bit rate, using 8000 as default value";
            return 8000;
        }
    }
}

quint32 RtspClientUdp::get_audio_channels()
{
    return audio_channels_;
}

QString & RtspClientUdp::get_audio_format()
{
    if (audio_format_ == "")
    {
        audio_format_ = "NONE";
    }
    return audio_format_;
}

QString & RtspClientUdp::get_sdp()
{
    return SDP_string;
}

quint32 RtspClientUdp::get_sdplen()
{
    return SDPlen;
}

QString & RtspClientUdp::get_audio_control()
{
    return acontrol_;
}

QString & RtspClientUdp::get_video_control()
{
    return vcontrol_;
}


/////////////////////////////////////////////////////////////////////
/// Donne nom et mot de passe
/// i Login_p    : nom
/// i Password_p : mot de passe
/////////////////////////////////////////////////////////////////////
void RtspClientUdp::SetCredentials(const QString& Login_p
                                   ,const QString& Password_p)
{
    Login_ = Login_p;
    Password_ = Password_p;
}


/////////////////////////////////////////////////////////////////////
/// Construit une ligne d'authentification
/// Si le nom ou le mot de passe manque, on obtient une chaîne vide.
/// Sinon, on obtient "Authorization: Basic XXX\r\n", où "XXX" est
/// le codage de "mon_nom:mon_mdp" en Base64.
/////////////////////////////////////////////////////////////////////
QString RtspClientUdp::Authentication() const
{
    QString Line;

    if (use_credentials_ && ! Login_.isEmpty() && ! Password_.isEmpty())
    {
        QString PlainCredentials(Login_ + ":" + Password_);

        QString Base64Credentials(PlainCredentials.toUtf8().toBase64());

        // Efface infos sensibles présentes en mémoire
        PlainCredentials = QString(PlainCredentials.length(),'X');

        Line = Header("Authorization","Basic " + Base64Credentials);

        // Efface infos sensibles présentes en mémoire
        Base64Credentials = QString(Base64Credentials.length(),'X');
    }
    return Line;
}


/////////////////////////////////////////////////////////////////////
/// Construit une ligne d'en-tête
/// i Name_p  : Nom de l'en-tête
/// i Value_p : valeur de l'en-tête
/// retour    : Ligne complète
/////////////////////////////////////////////////////////////////////
QString RtspClientUdp::Header(const QString &Name_p, const QString &Value_p) const
{
    return Name_p + ": " + Value_p + "\r\n";
}

/////////////////////////////////////////////////////////////////////
///
/////////////////////////////////////////////////////////////////////
void RtspClientUdp::CheckPtzCapabilities()
{
    const QString URL_p("/axis-cgi/admin/param.cgi?action=list&responseformat=rfc&group=Properties.PTZ");

    qDebug() << "RtspClientUdp::CheckPtzCapabilities";
    qDebug() << "serverName_:" << serverName_;
    qDebug() << "URL :       " << URL_p;

    if (PtzCapabilities_)
        return;

    HttpSock_.setHost(serverName_, 80);

    if (! Login_.isEmpty() && ! Password_.isEmpty())
    {
        qDebug() << "Login_:" << Login_;
        qDebug() << "Password_" << Password_;
        HttpSock_.setUser(Login_,Password_);
    }
    else
    {
        qDebug() << "Login en dur : root/omfgs5un";
        HttpSock_.setUser("root","omfgs5un");
    }

    connect(&HttpSock_, SIGNAL(requestFinished(int, bool)), this, SLOT(CheckPtzCapabilitiesAnalyse(int, bool)));

    Reply_to_check_ptz_.open(QBuffer::ReadWrite);

    HttpSock_.get(URL_p,&Reply_to_check_ptz_);
}

void RtspClientUdp::CheckPtzCapabilitiesAnalyse(int id, bool err)
{
    if (err)
    {
        qDebug() << "Reply_to_check_ptz, id = " << id << " err = " << err;
        return;
    }
    QString answer(Reply_to_check_ptz_.data().data());

    qDebug() << "Reply_to_check_ptz, data =" << answer;

    if (! answer.isEmpty())
    {
        if (answer.contains("=yes"))
        {
            PtzCapabilities_ = true;
	}
        qDebug() << "PTZ support = " << PtzCapabilities_;
        HttpSock_.disconnect();
        connect(&HttpSock_, SIGNAL(requestFinished(int, bool)), this, SLOT(httpRequestFinished(int, bool)));
    }
}

/////////////////////////////////////////////////////////////////////
///
/////////////////////////////////////////////////////////////////////
void RtspClientUdp::OnPtz(const QString & cmd)
{
    qDebug() << "RtspClientUdp::OnPtz: " << cmd;
    if (!PtzCapabilities_)
    {
        qDebug() << "RtspClientUdp::PTZ not supported by remote";
        return;
    }
     // PTZ camera locale
        qDebug() << "RtspClientUdp: send Ptz cmde to camera on port 80";
        int id = HttpSock_.get(cmd); // ignore reply value
        qDebug() << "GET, id =" << id << " url = " << cmd;
}

void RtspClientUdp::httpRequestFinished(int id, bool err)
{
    qDebug() << "RtspClientUdp::httpRequestFinished, id =" << id << " err=" << err;
}

void RtspClientUdp::watchdog()
{
#ifdef DEBUG_RTSP
    qDebug() << "watchdog" << watchdog_value_;
#endif
    if (watchdog_test_ == vlast_seq)
        watchdog_value_--;
    else
    {
        watchdog_value_ = WATCHDOG_TIMEOUT;
        watchdog_test_ = vlast_seq;
    }

    if (watchdog_value_ == 0)
    {
        emit error("watchdog has detected a problem !");
        reqSTOP();
    }
}

void RtspClientUdp::update_state(ClientState s)
{
#ifdef DEBUG_RTSP
    qDebug() << "--\nOld state : " << state_ << " New state : " << s << "\n--";
#endif
    state_ = s;
}
