/***************************************************************************
 *   Copyright (C) 2009, 2010 by Stephane List                             *
 *   slist@lilotux.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
///////////////////////////////////////////////////////////
/// sdp.h
/// Structure SDP - Interface
///////////////////////////////////////////////////////////

#if ! defined(SDP_H_INCLUDED_20080520_140616_)
#define SDP_H_INCLUDED_20080520_140616_

#pragma once

#include <vector>
#include <utility>

#include <QDebug>
#include <QString>

/////////////////////////////////////////////////////////////////////
/// Structures pour encapsuler une bande passante SDP (rfc2327)
/////////////////////////////////////////////////////////////////////
struct SDPBandwidth
{
	// b=<modifier>:<bandwidth-value>
	QString Modifier_;
	QString Value_;
	void Clear();
	QString toString() const;
};

/////////////////////////////////////////////////////////////////////
/// Structure pour encapsuler une connexion SDP (rfc2327)
/////////////////////////////////////////////////////////////////////
struct SDPConnection
{
	// c=<network type> <address type> <connection address>
	QString NetworkType_;
	QString AddressType_;
	QString ConnectionAddr_;
	void Clear();
	QString toString() const;
};

/////////////////////////////////////////////////////////////////////
/// Structure pour encapsuler une clef SDP (rfc2327)
/////////////////////////////////////////////////////////////////////
struct SDPKey
{
	// k=<method>:<encryption key>
	QString Method_;
	QString Value_;
	void Clear();
	QString toString() const;
};

/////////////////////////////////////////////////////////////////////
/// Structure pour encapsuler un temps SDP (rfc2327)
/////////////////////////////////////////////////////////////////////
struct SDPTime
{
	// t=<start time>  <stop time>
	long StartTime_;
	long StopTime_;

	// r=<repeat interval> <active duration> <list of offsets from start-time>
	long RepeatInterval_;
	long ActiveDuration_;
	std::vector<long> Offsets_;
	void Clear();
	QString toString() const;
};

typedef std::pair<QString,QString> SDPAttribute_t;
typedef std::vector<SDPAttribute_t> SDPAttributes_t;

/////////////////////////////////////////////////////////////////////
/// Structure pour encapsuler un média SDP (rfc2327)
/////////////////////////////////////////////////////////////////////
struct SDPMedia
{
	// m=<media> <port> <transport> <fmt list>
	QString Media_;
	QString Port_;
	QString Transport_;
	std::vector<QString> Formats_;

	// i=<session description>
	QString SessionDesc_;

	// c=<network type> <address type> <connection address>
	SDPConnection Cnx_;

	// b=<modifier>:<bandwidth-value>
	SDPBandwidth Bandwidth_;

	// k=<method>:<encryption key>
	SDPKey Key_;

	// a=<attribute>:<value>
	SDPAttributes_t Attributes_;
	void Clear();
	QString toString() const;
};

/////////////////////////////////////////////////////////////////////
/// Structure pour encapsuler une description SDP (rfc2327)
/////////////////////////////////////////////////////////////////////
class SDP
{
public:
	// v=0
	int Version_;

	// o=<username> <session id> <version> <network type> <address type> <address>
	QString UserName_;
	QString SessionId_;
	QString AnnounceVersion_;
	QString NetworkType_;
	QString AddressType_;
	QString Address_;

	// s=<session name>
	QString SessionName_;

	// i=<session description>
	QString SessionDesc_;

	// u=<URI>
	QString URI_;

	// e=<email address>
	QString EmailAddr_;

	// p=<phone number>
	QString PhoneNb_;

	// c=<network type> <address type> <connection address>
	SDPConnection Cnx_;

	// b=<modifier>:<bandwidth-value>
	SDPBandwidth Bandwidth_;

	// t=<start time>  <stop time>
	// r=<repeat interval> <active duration> <list of offsets from start-time>
	std::vector<SDPTime> Times_;

	// z=<adjustment time> <offset> <adjustment time> <offset> ....
	std::vector< std::pair<long,long> > Adjusts_;

	// k=<method>:<encryption key>
	SDPKey Key_;

	// a=<attribute>:<value>
	SDPAttributes_t Attributes_;

	std::vector<SDPMedia> Medias_;
	void Clear();
	QString toString() const;
};

QDebug operator<<(QDebug Out_p,const SDPBandwidth& BW_p);
QDebug operator<<(QDebug Out_p,const SDPConnection& Cnx_p);
QDebug operator<<(QDebug Out_p,const SDPTime& Time_p);
QDebug operator<<(QDebug Out_p,const SDPKey& Key_p);
QDebug operator<<(QDebug Out_p,const SDPMedia& Media_p);
QDebug operator<<(QDebug Out_p,const SDP& Info_p);

#endif // Garde d'inclusions multiples
