/***************************************************************************
 *   Copyright (C) 2009, 2010 by Stephane List                             *
 *   slist@lilotux.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//------------------------------------------------------------------------
/// utils.h : Divers utilitaires
//------------------------------------------------------------------------

#if ! defined(UTILS_H_20080625_135945_INCLUDED_)
#define UTILS_H_20080625_135945_INCLUDED_

#if defined(Q_OS_WIN32)
#include <windows.h>
#else
#include <unistd.h>
#endif

/////////////////////////////////////////////////////////////////////
/// Nombre d'éléments d'un tableau
/////////////////////////////////////////////////////////////////////
template<class T,const size_t N>
inline size_t ArrayCount(const T (&)[N])
{
	return N;
}

/////////////////////////////////////////////////////////////////////
/// Début d'un tableau
/////////////////////////////////////////////////////////////////////
template<class T,const size_t N>
inline T* ArrayBegin(const T (&Array)[N])
{
	return &Array[0];
}

/////////////////////////////////////////////////////////////////////
/// Fin d'un tableau
/////////////////////////////////////////////////////////////////////
template<class T,const size_t N>
inline T* ArrayEnd(const T (&Array)[N])
{
	return &Array[N];
}

//------------------------------------------------------------------------
/// Règle de trois (avec arrondi)
//------------------------------------------------------------------------
template<class T,class U,class V>
inline T MulDiv(T a,U b,V c)
{
	if (0 != c)
	{
		return static_cast<T>((a * b + c / 2) / c);
	}
	return 0;
}

#if defined(Q_OS_WIN32)

//------------------------------------------------------------------------
/// Pause en secondes (fonction Unix émulée sous Windows)
//------------------------------------------------------------------------
inline unsigned int sleep(unsigned int secs)
{
	::Sleep(secs * 1000);
	return 0;
}
#else

//------------------------------------------------------------------------
/// Pause en millisecondes (fonction Windows émulée sous Unix)
//------------------------------------------------------------------------
inline void Sleep(unsigned int millisecs)
{
	// usleep() est limité à 1 s, donc on fait une boucle.
	const unsigned int Granul = 500;

	while (millisecs > Granul)
	{
		usleep(Granul * 1000);
		millisecs -= Granul;
	}

	usleep(1000 * millisecs);
}

#endif

#endif
