/***************************************************************************
 *   Copyright (C) 2009, 2010 by Stephane List                             *
 *   slist@lilotux.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef _VIDEO_DECODER_H_
#define _VIDEO_DECODER_H_

#include <QtGui>

extern "C"
{
#include "libavcodec/avcodec.h"
#include "libswscale/swscale.h"
}

#include "video_decoder_h264.h"

/* How to use this class :

  1) instanciate the video decoder
  2) set parameters accoding to video format and SDP parameters
  3) init
  4) decode
  5) delete the object to release memory

  */

class QSize;
class QByteArray;

class VideoDecoder
{
public:

    VideoDecoder(const QString & videocodec);
    virtual ~VideoDecoder();
    void set_bitrate(int bitrate);
    bool init();

    /////////////////////////////////////////////////////////////////////
    /// Decoder
    /// i pBytes_p : Trame to decode
    /// i Size_p   : Nombre d'octets to decode
    /// i Format_p : Format de pixels souhait
    /// o PicSize_p: Dimensions de l'image
    /// o Video_p  : Trame decoded
    /// return : Success/fail
    /////////////////////////////////////////////////////////////////////
    bool Decode(const char* pBytes_p
                ,size_t Size_p
                ,int Format_p
                ,QSize& PicSize_p
                ,QByteArray& Video_p);

    void mp4_set_profile_id(int profile_id);
    void mp4_set_global_headers(QString & extradata);
    QString mp4_extradata_;

    void h264_set_packetization_mode(int);
    void h264_set_profile_level_id(int);
    void h264_set_sprop_parameter_sets(QString & sprop);
    PayloadContext pc_;

    QString video_format_;
    AVCodecContext* Grabber_;
    AVCodec* codec_;
    AVFrame* CodecFrame_;
    AVFrame* WindowFrame_;
    int count_;
    struct SwsContext* img_convert_ctx_;

};

#endif
